#!/usr/bin/python3
import asyncio
import client
import server

SETUP_HOST, SETUP_PORT = "0.0.0.0", 16660
SERVER_HOST, SERVER_PORT = "0.0.0.0", 16661
CHALLENGER_HOST, CHALLENGER_PORT = None, None

async def challenge_handler(reader, writer):
    print("[CHALLENGE] New setup")

    writer.write(b"Please provide a host and a port you are listening on (host:port): ")

    if not (received := (await reader.read(1024))):
        print("[CHALLENGE] Connection closed")
        return

    try:
        if received.count(b":") != 1:
            raise ValueError
        challenger_host, challenger_port = received.split(b":")
        challenger_host = challenger_host.decode()
        challenger_port = int(challenger_port)
        if (challenger_port < 1) | (challenger_port > 65535):
            raise ValueError
    except:
        writer.write(b"Invalid format, host or port. Closing connection.\n")
        await writer.drain()
        return

    print(f"[CHALLENGE] Challenger is supposed to be listening on {challenger_host}:{challenger_port}")

    connection_success = False
    # Attempt connection
    try:
        test_reader, test_writer = await asyncio.open_connection(challenger_host, challenger_port)
        test_writer.close()
        await test_writer.wait_closed()
        connection_success = True
    except:
        pass
    
    writer.write(b"MITM setup complete!\n")
    writer.write(f"The client will try to reach you at {challenger_host}:{challenger_port}\n".encode())
    writer.write(b"[CHALLENGE NOTE] - You don't have to wait longer than 10 seconds to see the first messages\n")
    writer.write(b"[CHALLENGE NOTE] - You don't have to wait longer than 5 minutes to see every type of message\n")
    if not connection_success:
        writer.write(b"[WARNING] - Unable to connect to the provided host and port! Please make sure your client is listening and exposed to the internet.\n")
    await writer.drain()

    try:
        if(client.RUNNING == True):
            await client.kill()

        challenger_addr = (challenger_host, challenger_port)
        await client.main(challenger_addr)
    except OSError:
        writer.write(b"Unexpected error setting up the challenge, it might already be running. Try restarting the instance. If the problem persists, please contact an administrator.\n")
        await writer.drain()

async def run_challenge():
    challenge_server = await asyncio.start_server(challenge_handler, SETUP_HOST, SETUP_PORT)
    async with challenge_server:
        await challenge_server.serve_forever()

async def main():
    await asyncio.gather(run_challenge(), server.run_server(SERVER_HOST, SERVER_PORT))

if(__name__ == "__main__"):
    asyncio.run(main())
