#!/usr/bin/python3
import util
import asyncio
from server_handlers import ServerHandler


async def server_handler(reader, writer):
    print("New server connection")

    handler = ServerHandler(reader, writer, SESSIONS)

    while True:
        if not (await handler.receive_message()):
            break

        session = SESSIONS[handler.session_id]
        await handler.handle(session)

    kill_list = []

    for s in SESSIONS:
        if(SESSIONS[s].expired()):
            print("Server session " + s.hex() + " expired")
            kill_list.append(s)
        else:
            print("Server session " + str(SESSIONS[s]) + " active")

    for kill in kill_list:
        del SESSIONS[kill]

async def run_server(host, port):
    server = await asyncio.start_server(server_handler, host, port)
    async with server:
        await server.serve_forever()

SESSIONS = {}
if(__name__ == "__main__"):
    asyncio.run(run_server("", 9999))
